import viz
import pylink

viz.go(viz.FULLSCREEN)
gallery = viz.add('gallery.ive')


#eyelink initialization.
SIZE = viz.window.getSize() #get the size of the window so that we can translate gaze data to viz coords.
RIGHT_EYE = 1
LEFT_EYE = 0
BINOCULAR = 2


eyelinktracker = pylink.EyeLink()

if not eyelinktracker:
	print "EL is None"
	sys.exit()
	

#Opens the EDF file.
edfFileName = "TEST.EDF";
eyelinktracker.openDataFile(edfFileName)		
	
pylink.flushGetkeyQueue(); 
eyelinktracker.setOfflineMode();                          

#Gets the display surface and sends a mesage to EDF file;
eyelinktracker.sendCommand("screen_pixel_coords =  0 0 %d %d" %(SIZE[0], SIZE[1]))
eyelinktracker.sendMessage("DISPLAY_COORDS  0 0 %d %d" %(SIZE[0], SIZE[1]))

tracker_software_ver = 0
eyelink_ver = eyelinktracker.getTrackerVersion()
if eyelink_ver == 3:
	tvstr = eyelinktracker.getTrackerVersionString()
	vindex = tvstr.find("EYELINK CL")
	tracker_software_ver = int(float(tvstr[(vindex + len("EYELINK CL")):].strip()))
	

if eyelink_ver>=2:
	eyelinktracker.sendCommand("select_parser_configuration 0")
	if eyelink_ver == 2: #turn off scenelink camera stuff
		eyelinktracker.sendCommand("scene_camera_gazemap = NO")
else:
	eyelinktracker.sendCommand("saccade_velocity_threshold = 35")
	eyelinktracker.sendCommand("saccade_acceleration_threshold = 9500")
	
# set EDF file contents 
eyelinktracker.sendCommand("file_event_filter = LEFT,RIGHT,FIXATION,SACCADE,BLINK,MESSAGE,BUTTON")
if tracker_software_ver>=4:
	eyelinktracker.sendCommand("file_sample_data  = LEFT,RIGHT,GAZE,AREA,GAZERES,STATUS,HTARGET")
else:
	eyelinktracker.sendCommand("file_sample_data  = LEFT,RIGHT,GAZE,AREA,GAZERES,STATUS")

# set link data (used for gaze cursor) 
eyelinktracker.sendCommand("link_event_filter = LEFT,RIGHT,FIXATION,SACCADE,BLINK,BUTTON")
if tracker_software_ver>=4:
	eyelinktracker.sendCommand("link_sample_data  = LEFT,RIGHT,GAZE,GAZERES,AREA,STATUS,HTARGET")
else:
	eyelinktracker.sendCommand("link_sample_data  = LEFT,RIGHT,GAZE,GAZERES,AREA,STATUS")
	
	

#load texture used for gaze cursor
gaze_cursor_texture=viz.add('crosshair.png')
gaze_cursor = viz.addTexQuad(viz.SCREEN,texture=gaze_cursor_texture)
gaze_cursor.visible(viz.OFF)



#stops recording on the tracker. onkeydown handler for 's'
def stopRecording():
	'''Ends recording: adds 100 msec of data to catch final events'''
	
	pylink.endRealTimeMode();  
	pylink.pumpDelay(100);       
	eyelinktracker.stopRecording();
	gaze_cursor.visible(viz.OFF)		 

		
#starts recording on the tracker. onkeydown handler for 'r'
def startRecording():
	if eyelinktracker.isRecording() ==0:
		return
	
		
	error = eyelinktracker.startRecording(1,1,1,1)
	if error:	return 
	#begin the realtime mode
	pylink.beginRealTimeMode(100)

	if not eyelinktracker.waitForBlockStart(100, 1, 0):
		stopRecording();
		print "ERROR: No link samples received!";

	gaze_cursor.visible(viz.ON)
		 





#collect the newest sample and draws the gaze cursor. ontimer event handler. 
def collectSample():
	if eyelinktracker.isRecording() ==0:
		dt = eyelinktracker.getNewestSample() # check for new sample update
		if(dt != None):
			eye_used = eyelinktracker.eyeAvailable()
			# Gets the gaze position of the latest sample,
			gaze_position = None
			if eye_used == RIGHT_EYE and dt.isRightSample():
				gaze_position = dt.getRightEye().getGaze()
			elif eye_used == LEFT_EYE and dt.isLeftSample():
				gaze_position = dt.getLeftEye().getGaze()
		
			
			if gaze_position is not None and gaze_position[0] != pylink.MISSING_DATA and gaze_position[1]!= pylink.MISSING_DATA:
				if gaze_cursor.getVisible() != viz.ON:
					gaze_cursor.visible(viz.ON)
				pos = ((gaze_position[0]/SIZE[0]),((SIZE[1]-gaze_position[1])/SIZE[1]))
				gaze_cursor.setPosition([pos[0],pos[1],1])
			else:
				gaze_cursor.visible(viz.OFF)
	
#stops recording, receive data file and closes the connection.
def closeConnection():
	stopRecording()
	eyelinktracker.setOfflineMode();                          
	pylink.msecDelay(500);                 

	#Close the file and transfer it to Display PC
	eyelinktracker.closeDataFile()
	eyelinktracker.receiveDataFile(edfFileName, edfFileName)
	eyelinktracker.close();
	
vizact.onkeydown('r', startRecording)
vizact.onkeydown('s', stopRecording)
vizact.ontimer(0.05,collectSample)
vizact.onexit(closeConnection)
